/* magicwrap.i --- SWIG directives for creating the interpreter-wrapped	*/
/* version of magic.							*/

%module magicwrap

%typemap(tcl8, ignore) int argc(int n_arg) {
    $1 = n_arg = objc - 1;
    objc = 2;
}

%typemap(tcl8, in) char **argv {
    int templength, i;
    $1 = (char **) mallocMagic(n_arg1 * sizeof(char *));
    for (i = 0; i < n_arg1; i++) {
	$1[i] = Tcl_GetStringFromObj(objv[i + 1], &templength);
    }
}

%typemap(tcl8, freearg) char **argv {
    freeMagic($1);
}

%{
#include <tk.h>

#include <unistd.h>
#include <signal.h>
#include <pthread.h>

#include "misc/magic.h"   
#include "utils/geometry.h"
#include "tiles/tile.h"  
#include "utils/hash.h"  
#include "utils/dqueue.h"
#include "database/database.h"
#include "windows/windows.h"
#include "commands/commands.h"
#include "utils/utils.h"
#include "textio/textio.h"
#include "textio/txcommands.h"
#include "drc/drc.h"
#include "signals/signals.h"
#include "graphics/graphics.h"
#include "utils/malloc.h" 

/*
 * String containing the version number of magic.  Don't change the string
 * here, nor its format.  It is updated by the Makefile in this directory.
 */

char *MagicVersion = MAGIC_VERSION;
char *MagicCompileTime = MAGIC_DATE;

Tcl_Interp *magicinterp;
Tcl_ThreadId drcthread;
extern int threadstatus;

/*----------------------------------------*/
/* Inline the two main wrapper prodedures */
/*----------------------------------------*/
 
static int _magic_start(ClientData clientData,
        Tcl_Interp *interp, int argc, char *argv[])
{
    /* Remember the interpreter */
    magicinterp = interp;
    threadstatus = 0;

    fprintf(stderr, "Starting magic under Tcl interpreter\n");
 
    mainInitBeforeArgs(argc, argv);
    mainDoArgs(argc, argv);
    mainInitAfterArgs();

    TxResetTerminal();	/* Give original properties back to stdin */

    return TCL_OK;
}

/*--------------------------------------------------------------*/
/* If DRCContinuous() is still working, set			*/
/* SigInterruptPending and wait for the thread to exit.	*/
/*--------------------------------------------------------------*/

void BreakDRC()
{
    
    if (threadstatus == 1)	/* thread is running */
    {
	GrFlush();

	/* fprintf(stderr, "Forcing break of DRC thread %d (wait on thread)\n", */
	/*	drcthread);							*/
	/* fflush(stderr);							*/

	/* Wait for EnableInterrupts and set SigInterruptPending	  */
	/* (should be calling Tcl functions, but no such functions exist) */
	pthread_kill((pthread_t)drcthread, SIGIO);
	pthread_join((pthread_t)drcthread, NULL); 

	SigIOReady = FALSE;
	SigInterruptOnSigIO = FALSE;
	SigInterruptPending = FALSE;

	/* fprintf(stderr, "DRC thread %d exited, continuing.\n", drcthread);	*/
	/* fflush(stderr);							*/
	/* threadstatus = -1; */ /* set by DRCContinuous() 			*/
    }
    else
	/* Keep track of nested BreakDRC calls */
	threadstatus--;
}

/*--------------------------------------------------------------*/
/* If the command result requires DRC to be done, restart	*/
/* the DRC thread.						*/
/*--------------------------------------------------------------*/

void ResumeDRC()
{
    if (((threadstatus == -1) || (threadstatus == 0)) && DRCHasWork)
    {
	/* fprintf(stderr, "Resuming (running) DRC thread\n");	*/
	/* fflush(stderr);					*/

	SigIOReady = FALSE;
	SigInterruptOnSigIO = TRUE;
	SigInterruptPending = FALSE;

	threadstatus = 1;	/* Thread running */

	Tcl_CreateThread(&drcthread, DRCContinuous, NULL,
		TCL_THREAD_STACK_DEFAULT, TCL_THREAD_JOINABLE);
	/* pthread_create(&drcthread, NULL, (void *)DRCContinuous, NULL); */
    }
    /* Keep track of nested BreakDRC returns */
    else if (threadstatus < 0) threadstatus++;
}

/*--------------------------------------------------------------*/
/* Deal with prompts and input focuse at the beginning of each	*/
/* command line.						*/
/*--------------------------------------------------------------*/

extern Void grTkRevertFocus();

static int _magic_prompt(ClientData clientData,
	Tcl_Interp *interp, int argc, char *argv[])
{
    Tcl_Channel outChannel;

    outChannel = Tcl_GetStdChannel(TCL_STDOUT);

    grTkRevertFocus();

    Tcl_WriteChars(outChannel, "> ", 2);

    return TCL_OK;
}

/*--------------------------------------------------------------*/
/* Dispatch a command from Tcl					*/
/*--------------------------------------------------------------*/

static int _tcl_dispatch(ClientData clientData,
        Tcl_Interp *interp, int argc, char *argv[])
{

    BreakDRC();
    TxTclDispatch(clientData, argc, argv);
    ResumeDRC();
    grTkRevertFocus();
    return TCL_OK;
}

/*--------------------------------------------------------------*/
/* Get a line from stdin (Tcl replacement for Tx function)	*/
/*--------------------------------------------------------------*/

char *
TxGetLine(dest, maxChars)
    char *dest;
    int maxChars;
{
    Tcl_Obj *objPtr;
    int charsStored, length;
    char *string;

    TxFlush();

    objPtr = Tcl_NewObj();
    charsStored = Tcl_GetsObj(Tcl_GetStdChannel(TCL_STDIN), objPtr);
    if (charsStored > 0) {
	string = Tcl_GetStringFromObj(objPtr, &length);
	if (length > maxChars) length = maxChars;
	strcpy(dest, string);
    }
    Tcl_DecrRefCount(objPtr);
    return dest;
}

/*--------------------------------------------------------------*/
/* Parse a file.  This is a skeleton version of the TxDispatch	*/
/* routine in textio/txCommands.c				*/
/*--------------------------------------------------------------*/

void
TxDispatch(f)
    FILE *f;	/* Under Tcl, we never call this with NULL */
{
    if (f == NULL)
    {
	fprintf(stderr, "Error:  TxDispatch(NULL) was called\n");
    }
    while (!feof(f))
    {
	if (SigInterruptPending)
	{
	    fprintf(stderr, "Read-in of file aborted.\n");
	    fflush(stderr);
	    SigInterruptPending = FALSE;
	    return;
	}
	txGetFileCommand(f, NULL);
    }
}

/*--------------------------------------------------------------*/
/* Send a command line which was collected by magic's TxEvent	*/
/* handler to the interpreter's event queue rather than sending	*/
/* it to TxParseString().					*/
/*--------------------------------------------------------------*/

void
TxParseString(str, q, event)
    char *str;
    caddr_t q;		/* unused */
    caddr_t event;	/* always NULL (ignored) */
{
    char *reply;

    Tcl_Eval(magicinterp, str);

    reply = Tcl_GetStringResult(magicinterp);
    if (strlen(reply) > 0)
	fprintf(stderr, "%s\n", reply);
}

%}

%init %{
{
    extern char *windCommands[];
    int n;
    char keyword[100];
    char *kwptr = keyword + 7;

    /* Use namespace to avoid conflicts with existing commands */
    strcpy(keyword, "magic::");

    for (n = 0; CmdLongCommands[n] != NULL; n++)
    {
	sscanf(CmdLongCommands[n], "%s ", kwptr); /* get first word */
	Tcl_CreateCommand(interp, keyword, _tcl_dispatch,
			(ClientData)NULL, (Tcl_CmdDeleteProc *) NULL);
    }
    for (n = 0; windCommands[n] != NULL; n++)
    {
	sscanf(windCommands[n], "%s ", kwptr); /* get first word */
	Tcl_CreateCommand(interp, keyword, _tcl_dispatch,
			(ClientData)NULL, (Tcl_CmdDeleteProc *) NULL);
    }

    /* Start command */
    Tcl_CreateCommand(interp, "magic::start", _magic_start,
			(ClientData)NULL, (Tcl_CmdDeleteProc *) NULL);

    /* Export the namespace commands */

    Tcl_Eval(interp, "namespace eval magic namespace export *");

    /* Add a "tcl_prompt1" variable pointint to the routing which */
    /* deals with prompt styles and input focus.		  */

   Tcl_CreateCommand(interp, "magic::prompt", _magic_prompt,
			(ClientData)NULL, (Tcl_CmdDeleteProc *) NULL);
   Tcl_SetVar(interp, "tcl_prompt1", "magic::prompt", TCL_GLOBAL_ONLY);
}
%}

/* %include "magicwrap.h" */
